Choice

Struct Choice 

pub struct Choice<A, B>(pub A, pub B);
Ordered choice combinator consuming/producing a sum type.

Parsing semantics: tries A first, wrapping success in Sum::Inl; on failure, tries B, wrapping success in Sum::Inr.

Consistency

If a value a is consistent with A, then Sum::Inl(a) is consistent with Choice(A, B). If a value b is consistent with B, then Sum::Inr(b) is consistent with Choice(A, B).

Unambiguity

Requires disjoint_domains(A, B).

Tuple Fields

0: A1: B