pub struct Dispatch<T, C, const N: usize>(pub T, pub [(T, C); N]);Expand description
Dispatch combinator that selects one of N branches based on a “tag” value.
Tuple Fields
0: T1: [(T, C); N]Implementations
Sourceimpl<T, C, const N: usize> Dispatch<T, C, N>
impl<T, C, const N: usize> Dispatch<T, C, N>
Sourcepub open spec fn has_active_branch(&self) -> bool
pub open spec fn has_active_branch(&self) -> bool
{ branch_exists(self.0, self.1@) }Sourcepub open spec fn active_branch(&self) -> C
pub open spec fn active_branch(&self) -> C
recommends
self.has_active_branch(),{ self.1[tag_position(self.0, self.1@) as int].1 }Sourcepub proof fn lemma_active_branch_is(&self, idx: nat)
pub proof fn lemma_active_branch_is(&self, idx: nat)
requires
idx < self.1@.len(),self.1[idx as int].0 == self.0,unique_branch_match(self.0, self.1@),ensuresself.has_active_branch(),self.active_branch() == self.1[idx as int].1,Trait Implementations
Sourceimpl<T, C: Consistency, const N: usize> Consistency for Dispatch<T, C, N>
impl<T, C: Consistency, const N: usize> Consistency for Dispatch<T, C, N>
Sourceopen spec fn consistent(&self, v: Self::Val) -> bool
open spec fn consistent(&self, v: Self::Val) -> bool
{
&&& self.has_active_branch()
&&& self.active_branch().consistent(v)
}Sourcetype Val = <C as Consistency>::Val
type Val = <C as Consistency>::Val
The type of values whose consistency is being checked.
Sourceimpl<T, C: EquivSerializers, const N: usize> EquivSerializers for Dispatch<T, C, N>
impl<T, C: EquivSerializers, const N: usize> EquivSerializers for Dispatch<T, C, N>
Sourceimpl<T, C, const N: usize> EquivSerializersGeneral for Dispatch<T, C, N>where
C: EquivSerializersGeneral,
impl<T, C, const N: usize> EquivSerializersGeneral for Dispatch<T, C, N>where
C: EquivSerializersGeneral,
Sourceopen spec fn equiv_general_inv(&self) -> bool
open spec fn equiv_general_inv(&self) -> bool
{ self.active_branch().equiv_general_inv() }Sourceproof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
proof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
Sourceimpl<T, C: GoodSerializer, const N: usize> GoodSerializer for Dispatch<T, C, N>
impl<T, C: GoodSerializer, const N: usize> GoodSerializer for Dispatch<T, C, N>
Sourceopen spec fn serialize_inv(&self) -> bool
open spec fn serialize_inv(&self) -> bool
{ self.active_branch().serialize_inv() }Sourceproof fn lemma_serialize_len(&self, v: Self::SVal)
proof fn lemma_serialize_len(&self, v: Self::SVal)
Sourceimpl<T, C: MinMaxByteLen, const N: usize> MinMaxByteLen for Dispatch<T, C, N>
impl<T, C: MinMaxByteLen, const N: usize> MinMaxByteLen for Dispatch<T, C, N>
Sourceimpl<T, C: NoLookAhead, const N: usize> NoLookAhead for Dispatch<T, C, N>
impl<T, C: NoLookAhead, const N: usize> NoLookAhead for Dispatch<T, C, N>
Sourceopen spec fn no_lookahead_inv(&self) -> bool
open spec fn no_lookahead_inv(&self) -> bool
{ self.active_branch().no_lookahead_inv() }Sourceproof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
proof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
Sourceimpl<T, C: NonMalleable, const N: usize> NonMalleable for Dispatch<T, C, N>
impl<T, C: NonMalleable, const N: usize> NonMalleable for Dispatch<T, C, N>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{ self.active_branch().nonmal_inv() }Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<T, C: NonTailFmt, const N: usize> NonTailFmt for Dispatch<T, C, N>
impl<T, C: NonTailFmt, const N: usize> NonTailFmt for Dispatch<T, C, N>
Sourceopen spec fn serialize_dps_inv(&self) -> bool
open spec fn serialize_dps_inv(&self) -> bool
{ self.active_branch().serialize_dps_inv() }Sourceproof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceproof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceimpl<T, C: Productive, const N: usize> Productive for Dispatch<T, C, N>
impl<T, C: Productive, const N: usize> Productive for Dispatch<T, C, N>
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{ self.active_branch().productive_inv() }Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<T, C, const N: usize> SPRoundTripDps for Dispatch<T, C, N>where
C: SPRoundTripDps,
impl<T, C, const N: usize> SPRoundTripDps for Dispatch<T, C, N>where
C: SPRoundTripDps,
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{ self.active_branch().unambiguous() }Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
Sourceimpl<T, C: SafeParser, const N: usize> SafeParser for Dispatch<T, C, N>
impl<T, C: SafeParser, const N: usize> SafeParser for Dispatch<T, C, N>
Sourceimpl<T, C: SoundParser, const N: usize> SoundParser for Dispatch<T, C, N>
impl<T, C: SoundParser, const N: usize> SoundParser for Dispatch<T, C, N>
Sourceimpl<T, C: SpecByteLen, const N: usize> SpecByteLen for Dispatch<T, C, N>
impl<T, C: SpecByteLen, const N: usize> SpecByteLen for Dispatch<T, C, N>
Sourceimpl<T, C: SpecParser, const N: usize> SpecParser for Dispatch<T, C, N>
impl<T, C: SpecParser, const N: usize> SpecParser for Dispatch<T, C, N>
Sourceimpl<T, C: SpecSerializer, const N: usize> SpecSerializer for Dispatch<T, C, N>
impl<T, C: SpecSerializer, const N: usize> SpecSerializer for Dispatch<T, C, N>
Sourceopen spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
{ self.active_branch().spec_serialize(v) }Sourcetype SVal = <C as SpecSerializer>::SVal
type SVal = <C as SpecSerializer>::SVal
The type of values to be serialized.
Sourceimpl<T, C: SpecSerializerDps, const N: usize> SpecSerializerDps for Dispatch<T, C, N>
impl<T, C: SpecSerializerDps, const N: usize> SpecSerializerDps for Dispatch<T, C, N>
Sourceimpl<T, C: StaticByteLen, const N: usize> StaticByteLen for Dispatch<T, C, N>
impl<T, C: StaticByteLen, const N: usize> StaticByteLen for Dispatch<T, C, N>
Sourceopen spec fn static_byte_len() -> nat
open spec fn static_byte_len() -> nat
{ C::static_byte_len() }Sourceproof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_static_len_matches_byte_len(&self, v: Self::T)
Sourceimpl<T, C: ValueByteLen, const N: usize> ValueByteLen for Dispatch<T, C, N>
impl<T, C: ValueByteLen, const N: usize> ValueByteLen for Dispatch<T, C, N>
Sourceopen spec fn value_byte_len(v: Self::T) -> nat
open spec fn value_byte_len(v: Self::T) -> nat
{ C::value_byte_len(v) }Sourceproof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_value_len_matches_byte_len(&self, v: Self::T)