pub struct Alt<A, B, const NONDETERMINISTIC: bool = false>(pub A, pub B);Expand description
Ordered alternative combinator.
Parsing semantics: like Choice, but both branches consume/produce the same type.
The result is returned directly without a Sum wrapper.
Serialization semantics: if only one branch is consistent with a value, that branch is used. If both branches are consistent, serialization is intentionally underspecified and may use either branch.
Consistency
A value v is consistent with Alt(A, B) iff it is consistent with A OR B.
Unambiguity
Requires disjoint_domains(A, B).
Malleability
This combinator introduces malleability by default.
Non-malleability can be recovered if A is disjoint from B (see disjoint_values).
Tuple Fields
0: A1: BImplementations
Sourceimpl<const NONDETERMINISTIC: bool, A, B> Alt<A, B, NONDETERMINISTIC>
impl<const NONDETERMINISTIC: bool, A, B> Alt<A, B, NONDETERMINISTIC>
Sourcepub open spec fn choose_left(&self, v: A::Val) -> bool
pub open spec fn choose_left(&self, v: A::Val) -> bool
{
if NONDETERMINISTIC {
arbitrary_or_left(self.0.consistent(v), self.1.consistent(v))
} else {
self.0.consistent(v)
}
}For non-deterministic Alt:
If exactly one branch accepts v, this returns that branch.
If both accept v, the choice is unspecified.
For deterministic Alt, this returns true iff the left branch accepts v.